/* Arduino Nofrendo
 * Please check hw_config.h and display.cpp for configuration details
 * 
 * 2024'11/30 Yuji Tanaka
 *   Support 40 *.nes files.
 *   Support I2S and ILI9163 LCD panel.
 *   Modify for GS-0005 board.
 */
#include <esp_wifi.h>
#include <esp_task_wdt.h>
#include <FFat.h>
#include <SPIFFS.h>
#include <SD.h>
#include <SD_MMC.h>

#include <Arduino_GFX_Library.h>

#include "hw_config.h"

extern "C"
{
#include <nofrendo.h>
}
int16_t bg_color;
extern Arduino_TFT *gfx;
extern void display_begin();
extern "C" void controller_init();
extern "C" uint32_t controller_read_input();

#define MAX_FILE 40
char fname[MAX_FILE][64];
char old_fname[64];
int fnum;
int sel, old_sel;

void setup()
{
    Serial.begin(115200);

    // turn off WiFi
    esp_wifi_deinit();

    // disable Core 0 WDT
    TaskHandle_t idle_0 = xTaskGetIdleTaskHandleForCPU(0);
    esp_task_wdt_delete(idle_0);

    // start display
    display_begin();

    // filesystem defined in hw_config.h
    FILESYSTEM_BEGIN

    // find first rom file (*.nes)
    File root = filesystem.open("/");
    char *argv[1];
    if (!root)
    {
        Serial.println("Filesystem mount failed! Please check hw_config.h settings.");
        gfx->println("Filesystem mount failed! Please check hw_config.h settings.");
    }
    else
    {
        bool foundRom = false;

        fnum=0;
        strcpy(old_fname, "/");
        printDirectory(root, 0, ".nes");
        if (fnum>0)
            foundRom = true;

        if (!foundRom)
        {
            Serial.println("Failed to find rom file, please copy rom file to data folder and upload with \"ESP32 Sketch Data Upload\"");
            argv[0] = "/";
        }

        sel=0;
        old_sel=-1;
        controller_init();
        select_file();
        
        char fullFilename[256];
        sprintf(fullFilename, "%s/%s", FSROOT, fname[sel]);
        argv[0] = fullFilename;

        Serial.println("NoFrendo start!\n");
        nofrendo_main(1, argv);
        Serial.println("NoFrendo end!\n");
    }
}

void loop()
{
}

void printDirectory(File dir, int numTabs, char *typestr) {
  int i;
  while (true) {

    File entry =  dir.openNextFile();
    if (! entry) {
      // no more files
      for(i=strlen(old_fname)-2;i>=0;i--){
        if (old_fname[i]=='/'){
          old_fname[i+1]='\0';
          break;
        }
      }
      break;
    }
    if (fnum<MAX_FILE && stristr(entry.name(), typestr)!=NULL){
      strcpy(fname[fnum], old_fname);
      strcat(fname[fnum], entry.name());
      Serial.println(fname[fnum]);
      fnum++;
    }else if(fnum>=MAX_FILE){
      entry.close();
      return;
    }
    
    if (entry.isDirectory()) {
      strcat(old_fname, entry.name());
      strcat(old_fname, "/");
      // Serial.println("/");
      printDirectory(entry, numTabs + 1, typestr);
    }
    entry.close();
  }
}

char *stristr(const char *org, char *str)
{
  char *tmp, *tmp0, *tmp1;
  tmp=(char *)org;
  for(;*tmp!='\0';tmp++){
    tmp0=tmp;
    tmp1=str;
    for(;*tmp1!='\0';tmp1++){
      if (*tmp0==*tmp1){
        tmp0++;
        continue;
      }else if ('A'<=*tmp1 && *tmp1<='Z' && *tmp0==(*tmp1-'A'+'a')){
        tmp0++;
        continue;
      }else if ('a'<=*tmp1 && *tmp1<='z' && *tmp0==(*tmp1-'a'+'A')){
        tmp0++;
        continue;
      }else
        break;
    }
    if (*tmp1=='\0')
      return(tmp);
  }
  return(NULL);
}

int select_file()
{
  int i;
  uint32_t key, old_key;

  gfx->begin();
  gfx->fillScreen(BLACK);
  
  old_key=0;
  
  for(;;){
    key = controller_read_input();
    if (key==old_key){
      delay(50);
      continue;
    }
    
    old_key=key;
    
    if ((key & 64)==0 || (key & 128)==0)
      break;
 
    if ((key & 1)==0)
      sel--;
    if ((key & 2)==0)
      sel++;
    if (sel<0)
      sel=fnum-1;
    if (sel>=fnum)
      sel=0;

    if (sel!=old_sel){
      for(i=0;i<fnum;i++){
        gfx->setCursor(0, i*8);
        if (i==sel)
          gfx->setTextColor(RED);
        else
          gfx->setTextColor(WHITE);
        gfx->print(fname[i]);
      }
      old_sel=sel;
    }
    
    delay(50);
  }
  return(sel);
}
